<?php
/*
 *
 * Cette classe permet de grer les diffrentes collectivits.
 * Les mthodes de bases sont hrites de DataObject.
*/

require_once("DataObject.class.php");
require_once("Module.class.php");

class Authority extends DataObject {
  protected $objectName = "authorities";
  protected $prettyName = "Collectivit";

  protected $name;
  protected $siren;
  protected $authority_group_id;
  protected $agreement;
  protected $email;
  protected $broadcast_email;
  protected $default_broadcast_email;
  protected $authority_type_id;
  protected $status;
  protected $address;
  protected $postal_code;
  protected $city;
  protected $department;
  protected $district;
  protected $telephone;
  protected $fax;

  private $modulesPerms = null;

  private $authorityTypeCode = null;
  private $authorityTypeDescr = null;

  protected $dbFields = array( "name" => array( "descr" => "Nom", "type" => "isString", "mandatory" => true),
						 "siren" => array( "descr" => "Numro de SIREN", "type" => "isString", "mandatory" => true, "unique" => true),
						 "authority_group_id" => array( "descr" => "Groupe de collectivit", "type" => "isInt", "mandatory" => false),
						 "agreement" => array( "descr" => "Rfrence convention", "type" => "isString", "mandatory" => false),
						 "email" => array( "descr" => "Adresse lectronique", "type" => "isEmail", "mandatory" => false),
						 "broadcast_email" => array( "descr" => "Adresse lectronique de diffusion", "type" => "isString", "mandatory" => false),
						 "default_broadcast_email" => array( "descr" => "Adresse lectronique de diffusion par dfaut", "type" => "isString", "mandatory" => false),
						 "status" => array( "descr" => "tat", "type" => "isInt", "mandatory" => true),
						 "authority_type_id" => array( "descr" => "Type de collectivit", "type" => "isString", "mandatory" => true),
						 "address" => array( "descr" => "Adresse", "type" => "isString", "mandatory" => true),
						 "postal_code" => array( "descr" => "Code postal", "type" => "isString", "mandatory" => true),
						 "city" => array( "descr" => "Ville", "type" => "isString", "mandatory" => true),
						 "department" => array( "descr" => "Dpartement", "type" => "isString", "mandatory" => true),
						 "district" => array( "descr" => "Arrondissement", "type" => "isString", "mandatory" => true),
						 "telephone" => array( "descr" => "Tlphone", "type" => "isString", "mandatory" => false),
						 "fax" => array( "descr" => "Fax", "type" => "isString", "mandatory" => false),
	  					"ext_siret" =>array("descr"=> "ext_SIRET","type" =>"isString", "mandatory"=>false),
	  					"helios_ftp_login" =>array("descr"=>"helios_ftp_login","type"=>"isString","mandatory" =>false),
	  					"helios_ftp_password" =>array("descr" => "helios_ftp_password", "type" => "isString", "mandatory" =>false),
	    				"helios_ftp_dest" =>array("descr" => "helios_ftp_dest", "type" => "isString", "mandatory" =>false),
	  					"email_mail_securise" => array("descr" => "Email pour le module de mail scuris", "type" => "isString", "mandatory" =>false),
						"new_notification" => array("descr" => "Nouveau systeme de notification", "type" => "boolean", "mandatory" =>false),
  						"dia_siret" => array( "descr" => "Numro de SIRET de rception des DIA", "type" => "isString", "mandatory" => false, "unique" => true),
  						"sae_wsdl" => array("descr"=>'','type'=>'isString','mandatory'=>false,'unique'=>false),
						 );


  /**
   * \brief Constructeur d'une collectivit
   * \param id integer Numro d'id d'une collectivit existante avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
    parent::__construct($id);
	if ($id) {
	  $this->init();
	  $this->initModulesPerms();
	}
  }

  /**
   * \brief Mthode d'initialisation d'une collectivit depuis la base de donnes
   * \return true si succs, false sinon
  */
  public function init() {
	return (parent::init() && $this->initModulesPerms());
  }

  /**
   * \brief Mthode d'obtention du type de collectivit sous forme de chane
   * \param $only_descr boolen (optionnel) : ne renvoie que la description si positionn  true (false par dfaut)
   * \return Une chane indiquant le type de la collectivit
   */
  public function getAuthorityTypeName($only_descr = false) {
	if (! $this->authorityTypeCode ||!$this->authorityTypeDescr) {
	  if (isset($this->authority_type_id)) {
		$sql = "SELECT id, description FROM authority_types WHERE id=" . $this->authority_type_id;

		$result = $this->db->select($sql);

		if ($result->isError()) {
		  $this->errorMsg = "Erreur lors de la rcupration du type de collectivit.";
		  return null;
		} else {
		  $row = $result->get_next_row();
		  $this->authorityTypeCode = $row["id"];
		  $this->authorityTypeDescr = $row["description"];
		}
	  }
	}

	if ($only_descr) {
	  return $this->authorityTypeDescr;
	} else {
	  return $this->authorityTypeCode . "&nbsp;-&nbsp;" . $this->authorityTypeDescr;
	}
  }

  /**
   * \brief Mthode d'obtention du dpartement et de l'arrondissement d'appartenance de la collectivit
   * \return Une chane indiquant le dpartement et l'arrondissement
   */
  public function getDeptDistrString() {
	if (! empty($this->department) && ! empty($this->district)) {
	  $str = "";

	  $sql = "SELECT id, name FROM authority_departments WHERE code='" . $this->department . "'";

	  $result = $this->db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() == 1) {
		$row = $result->get_next_row();
		$str .= $row["name"];
		$deptId = $row["id"];
	  } else {
		$str .= $this->department;
		$deptId = null;
	  }

	  $str .= "&nbsp;/&nbsp;";

	  $sql = "SELECT name FROM authority_districts WHERE code='" . $this->district . "'";

	  if ($deptId) {
		$sql .= " AND authority_department_id=" . $deptId;
	  }

	  $result = $this->db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() == 1) {
		$row = $result->get_next_row();
		$str .= $row["name"];
	  } else {
		$str .= $this->district;
	  }

	  return $str;
	}

	return null;
  }

  /**
   * \brief Mthode qui renvoie la liste des modules autoriss pour cette collectivit
   */
  public function getAuthorizedModules() {
	if (! $this->modulesPerms) {
	  $this->initModulesPerms();
	}

	return $this->modulesPerms;
  }

  /**
   * \brief Mthode qui dtermine si la collectivit appartient au groupe spcifi
   * \param $group_id integer : Numro d'identifiant du groupe
   * \return True si la collectivit appartient au groupe, False sinon
  */
  public function isInGroup($group_id) {
	if (is_numeric($group_id)) {
	  if ($this->authority_group_id == $group_id) {
		return true;
	  }
	}

	return false;
  }

  /**
   * \brief Mthode d'initialisation des permissions de la collectivit sur les modules
  */
  public function initModulesPerms() {
	if (isset($this->id)) {
	  $this->modulesPerms = Module::getModulesForAuthority($this->id);
	  return true;
	}

	return false;
  }

  /**
   * \brief Mthode de remise  zro des permissions sur les modules
  */
  public function resetModulesPerms() {
	$this->modulesPerms = null;
  }

  /**
   * \brief Mthode qui permet de fixer les permissions d'une collectivit sur un module
   * \param $module_id integer Numro identifiant le module sur lequel fixer la permission
   * \param $val boolen ou chaine (optionnel) Valeur de la permission, true par dfaut, peut tre "1" ou "on" ou true pour activer la permission
  */
  public function setModulePerm($module_id, $val = true) {
	if ($val == "1"||$val == "on"||$val === true) {
	  $this->modulesPerms[$module_id] = true;
	}
  }

  /**
   * \brief Mthode d'obtention de la permission d'une collectivit sur un module
   * \param $module_id integer Numro d'identifiant du module
   * \return true si autorise, false sinon
  */
  public function getModulePerm($module_id) {
	if (! $this->modulesPerms) {
	  $this->getAuthorizedModules();
	}

	if (isset($this->modulesPerms[$module_id])) {
	  return $this->modulesPerms[$module_id];
	} else {
	  return false;
	}
  }
  
  public function getModulePermByName($module_name)
  {
  	$sql = "SELECT id FROM modules WHERE name='".$module_name."'";

    $db = DatabasePool::getInstance();
	
    $result = $db->select($sql);
    if (! $result->isError()) {
    	$row=$result->get_next_row();
    	$id=$row["id"];
    	return $this->getModulePerm($id);
    }
    else 
    	return false;
  }
  /**
   * \brief Mthode permettant de savoir si une collectivit est active
   * \return true si active, false sinon
  */
  public function isActive() {
    return ($this->status == 1);
  }

  /**
   * \brief Mthode d'enregistrement d'une collectivit dans la base de donnes
   * \param $module_perms (optionnel) : si true (dfaut) sauvegarde aussi les permissions sur les modules
   * \param $validate boolen (optionnel) Prcise si la validation de l'entit doit avoir lieu (true par dfaut)
   * \return true si succs, false sinon
  */
  public function save($module_perms = true, $validate = true) {
    if (! ($sql = parent::save($validate, true))) {
	  return false;
	}

    //echo $sql;
    //exit();

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde de la collectivit.";
	  $this->db->rollback();
      return false;
    }

	if ($module_perms) {
	  // Traitement permissions sur les modules
	  $sql = "DELETE FROM modules_authorities WHERE authority_id=" . $this->id;

	  if (! $this->db->exec($sql)) {
		$this->errorMsg = "Erreur lors de la rinitialisation des permissions de la collectivit.";
		$this->db->rollback();
		return false;
	  }

	  if (count($this->modulesPerms) > 0) {
		reset($this->modulesPerms);
		foreach ($this->modulesPerms as $module_id => $val) {
		  $sql = "INSERT INTO modules_authorities (module_id, authority_id) VALUES(" . $module_id . ", " . $this->id . ")";
		  
		  if (! $this->db->exec($sql)) {
			$this->errorMsg = "Erreur lors de la sauvegarde des permissions de la collectivit.";
			$this->db->rollback();
			return false;
		  }
		}
	  }
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

    return true;
  }

  /**
   * \brief Mthode permettant de supprimer une collectivit de a base de donnes
   * \param $id integer (optionnel) Numro d'identifiant de la collectivit, si non spcifi, collectivit en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && ! empty($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

	$sql = "DELETE FROM modules_authorities WHERE authority_id=" . $id;

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression des associations avec les modules.";
	  $this->db->rollback();
	  return false;
    }

	if (! parent::delete($id)) {
	  $this->db->rollback();
      return false;
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

	return true;
  }

  /**
   * \brief Mthode d'obtention de la liste des collectivits et tous leurs attributs
   * \param $cond (optionnel) chane Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \return tableau des collectivits
  */
  public function getAuthoritiesList($cond = "") {
	if (! $this->pagerInit('authorities.id, authorities.name, authorities.siren, authorities.authority_group_id, authorities.agreement, authorities.email, authorities.broadcast_email, authorities.authority_type_id, authority_types.description AS type_name, authorities.address, authorities.postal_code, authorities.city, authorities.telephone, authorities.fax',
						 'authorities LEFT JOIN authority_types ON authorities.authority_type_id=authority_types.id',
						 $cond,'authorities.name',null,null,"ASC")) {
	  return false;
	}

    return $this->data;
  }
  
  
  public function getAllAuthorities(){
  	return $this->db->fetchAll("SELECT id,name,siren FROM authorities");
  }
  

  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode d'obtention d'une liste de collectivit
   * \param $cond chaine : condition SQL a appliquer sur la requete
   * \return Tableau de collectivits
   *
   * Cette mthode retourne un tableau dont les clefs sont les identifiants
   * des collectivits et le contenu de la case est le nom de la collectivit
  */
  public static function getAuthoritiesIdName($cond = "") {
    $sql = "SELECT authorities.id, authorities.name FROM authorities " . $cond;

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

    if (! $result->isError()) {
      $authorities = $result->get_all_rows();
    }

    $tabAuthorities = array();
	if (count($authorities) > 0) {
	  foreach ($authorities as $key => $authority) {
		$tabAuthorities[$authority["id"]] = $authority["name"];
	  }
	}

    return $tabAuthorities;
  }
  
  public static function getSirenFromId($id){
    $sql = "SELECT siren FROM authorities WHERE id = ".$id;

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

    if (!$result->isError() && $result->num_row() == 1) {
      $row = $result->get_next_row();
      return $row["siren"];
    }

    return false;

  }
  

  /**
   * \brief Mthode d'obtention de la liste id/name des types de collectivits
   * \return Tableau des types de collectivits
   *
   * Cette mthode retourne un tableau dont les clefs sont les identifiants
   * des types et le contenu de la case est le nom du type (tronqu  40 caractres)
   *
  */
  public static function getAuthorityTypesIdName() {
	// Obtention des types parents
	$sql = "SELECT id, parent_type_id, description FROM authority_types where parent_type_id is not null";

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

	$types = array();

    if (! $result->isError()) {
	  while ($row = $result->get_next_row()) {
		$types[$row["id"]] = $row["id"] . ' - ' . substr($row["description"], 0, 40);

		if (strlen($row["description"]) > 40) {
		  $types[$row["id"]] .= "...";
		}
	  }
	}

	return $types;
  }

  /**
   * \brief Mthode d'obtention de la liste des types de collectivits
   * \return Tableau des types de collectivits
  */
  public static function getAuthorityTypes() {
	// Obtention des types parents
	$sql = "SELECT id, parent_type_id, description FROM authority_types WHERE parent_type_id is null";

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

	$types = array();

    if (! $result->isError()) {
	  while ($row = $result->get_next_row()) {
		$types[] = array("code" => $row["id"], "description" => $row["description"], "type" => "parent");

		$sql = "SELECT id, parent_type_id, description FROM authority_types WHERE parent_type_id=" . $row["id"];

		$result2 = $db->select($sql);
		if (! $result2->isError()) {
		  while ($row2 = $result2->get_next_row()) {
			$types[] = array("code" => $row2["id"], "description" => $row2["description"], "type" => "child");
		  }
		}
	  }
    }

    return $types;
  }

  /**
   * \brief Mthode d'obtention d'une liste id/nom de dpartements
   * \return Tableau de dpartements
   *
   * Cette mthode retourne un tableau dont les clefs sont les identifiants
   * des dpartements et le contenu de la case est le nom du dpartement
  */
  public static function getDepartmentsIdName() {
    $sql = "SELECT code, name FROM authority_departments";

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

	$departments = array();

    if (! $result->isError()) {
      while ($row = $result->get_next_row()) {
		$departments[$row["code"]] = $row["name"];
	  }
    }

    return $departments;
  }

  /**
   * \brief Mthode d'obtention d'une liste de dpartement
   * \return Tableau de dpartements
   *
  */
  public static function getDepartmentsList() {
    $sql = "SELECT id, code, name FROM authority_departments";

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

	$departments = array();

    if (! $result->isError()) {
      return $result->get_all_rows();
    }

    return array();
  }


  /**
   * \brief Mthode d'obtention d'une liste d'arrondissements pour un dpartement donn
   * \return Tableau d'arrondissements
   *
   * Cette mthode retourne un tableau dont les clefs sont les identifiants
   * des arrondissements et le contenu de la case est le nom de l'arrondissement
  */
  public static function getDistrictsForDepartment($dept) {
	$districts = array();

	if (isset($dept) && ! empty($dept)) {
	  $sql = "SELECT authority_districts.code, authority_districts.name FROM authority_districts LEFT JOIN authority_departments ON authority_districts.authority_department_id=authority_departments.id WHERE authority_departments.code='" . $dept . "'";

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $districts[$row["code"]] = $row["name"];
		}
	  }
	}
	
	return $districts;
  }
  
}
