<?php
require_once("../../../config/config.php");
require_once(SITEROOT . '/class/include.class.php');

$me = new User();

if (! $me->authenticate()) {
  $_SESSION["error"] = "chec de l'authentification";
  header("Location: " . WEBSITE);
  exit();
}

if (! $me->isGroupAdminOrSuper()) {
  $_SESSION["error"] = "Accs refus";
  header("Location: " . WEBSITE_SSL);
  exit();
}

$ftype =  Helpers::getVarFromGet("type");
$fname = Helpers::getVarFromGet("name");
$fgroup = Helpers::getVarFromGet("group");
$api = Helpers::getVarFromGet("api");
$fsiren = Helpers::getVarFromGet("siren");

$authority = new Authority();

$filter = array();
if ($me->isGroupAdmin()) {
  $filter[] .= "authorities.authority_group_id=" . addslashes($me->get("authority_group_id"));
}

if (isset($ftype) && strlen($ftype) > 0) {
  $filter[] .= "authorities.authority_type_id='" . addslashes($ftype) . "'";
}

if (isset($fname) && strlen($fname) > 0) {
  $filter[] .= "authorities.name ILIKE '%" . addslashes($fname) . "%'";
}

if (isset($fgroup) && is_numeric($fgroup)) {
  $filter[] .= "authorities.authority_group_id=" . addslashes($fgroup);
}

if (isset($fsiren) && is_numeric($fsiren)) {
  $filter[] .= "authorities.siren ILIKE '%" . addslashes($fsiren) ."%'";
}

$where = "";
if (count($filter) > 0) {
  $where = "WHERE " . implode($filter, " AND ");
}

$authorities = $authority->getAuthoritiesList($where);

if ($api){
	$jsonOutput->retrictAndDisplay($authorities,array('id','name','authority_group_id','siren','address','city','postal_code','telephone'));
	exit;
}


$doc = new HTMLLayout();
$doc->setTitle("Tedetis : gestion des collectivits");
$doc->openContainer();
$doc->openSideBar();
$doc->buildMenu($me);
$doc->buildPager($authority);
$doc->closeSideBar();
$doc->openContent();

$html = "<h1>Gestion des collectivits";

if ($me->isGroupAdmin()) {
  $myGroup = new Group($me->get("authority_group_id"));
  $html .= " du groupe " . htmlspecialchars($myGroup->get("name"));
}

$html .= "</h1>\n";
$html .= "<div id=\"actions-area\">\n";
$html .= "<h2>Actions</h2>";
$html .= "<a class=\"btn btn-primary\" href=\"" . WEBSITE_SSL . "/admin/authorities/admin_authority_edit.php\" class=\"bouton\">Ajouter une collectivit</a>\n";
$html .= "</div>\n";
$html .= "<div id=\"filtering-area\">\n";
$html .= "<h2>Filtrage</h2>\n";
$html .= "<form class=\"form-horizontal\" action=\"admin_authorities.php\" method=\"get\" role=\"form\">\n";
$html .= "<div class=\"form-group\">\n";
$html .= "<label for=\"type\" class=\"col-md-3 control-label\">Type</label>\n";
$html .= "<div class=\"col-md-3\">" . $doc->getHTMLSelect("type", Authority::getAuthorityTypesIdName(), $ftype) . "</div>\n";
$html .= "<label for=\"name-contain\" class=\"col-md-3 control-label\">Le nom contient</label>\n";
$html .= "<div class=\"col-md-3\"><input id=\"name-contain\" class=\"form-control\" type=\"text\" name=\"name\" size=\"20\" maxlength=\"25\"";

if (strlen($fname) > 0) {
  $html .= " value=\"" . htmlspecialchars($fname) . "\"";
}

$html .= " /></div>\n";
$html .= "</div>\n";

if ($me->isSuper()) {
  $html .= "<div class=\"form-group\">\n";
  $html .= "<label for=\"group\" class=\"col-md-3 control-label\">Groupe</label>\n";
  $html .= "<div class=\"col-md-3\">" . $doc->getHTMLSelect("group", Group::getGroupsIdName(), $fgroup) . "</div>\n";
  $html .= "</div>\n";
}
$html .= "<div class=\"form-group\">\n";
$html .= "<button class=\"btn btn-default col-md-offset-3 col-md-3\" type=\"submit\">Filtrer</button>\n";
$html .= "</div>\n";
$html .= "</form>\n";
$html .= "</div><br />\n";
$html .= "<h2>Liste des collectivits</h2>\n";
$html .= "<div id=\"authority-list\">\n";
$html .= "<table class=\"data-table table table-striped\" summary=\"\">";
$html .= "<thead>\n";
$html .= "<tr>\n";
$html .= " <th id=\"name\">Nom</th>\n";
$html .= " <th id=\"group-member\">Groupe</th>\n";
$html .= " <th id=\"authority-type\">Type de collectivit</th>\n";
$html .= " <th id=\"address\">Adresse</th>\n";
$html .= " <th id=\"phone\">Tlphone</th>\n";
$html .= " <th id=\"fax\">Fax</th>\n";
$html .= " <th id=\"actions\">Actions</th>\n";
$html .= "</tr>\n";
$html .= "</thead>\n";
$html .= "<tbody>\n";

$i = 0;

foreach ($authorities as $ent) {
  $group = new Group($ent["authority_group_id"]);

  if (strlen($group->get("name")) > 0) {
	$groupName = $group->get("name");
  } else {
	$groupName = "Aucun";
  }

  $html .= "<tr>\n";
  $html .= " <td headers=\"name\">" . $ent["name"] . "</td>\n";
  $html .= " <td headers=\"group-member\">" . $groupName . "</td>\n";
  $html .= " <td headers=\"authority-type\">" . $ent["type_name"] . "</td>\n";
  $html .= " <td headers=\"address\" class=\"long_field\">" . nl2br($ent["address"]) . "<br />" . $ent["postal_code"] . " " . $ent["city"] . "</td>\n";
  $html .= " <td headers=\"phone\">" . $ent["telephone"] . "</td>\n";
  $html .= " <td headers=\"fax\">" . $ent["fax"] . "</td>\n";
  $html .= " <td headers=\"actions\"><a href=\"admin_authority_edit.php?id=" . $ent["id"] . "\" class=\"icon\"><img src=\"" . WEBSITE_SSL . "/custom/images/erreur.png\" alt=\"image_modif\" title=\"Modifier\" /></a></td>\n";
  $html .= "</tr>\n";

}

$html .= "</tbody>\n";
$html .= "</table>\n";
$html .= "</div>\n";
$html .= "</div>\n";

$doc->addBody($html);

$doc->closeContent();
$doc->closeContainer();

$doc->buildFooter();

$doc->display();
