<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
?>
<?php
/**
 * \class DataObject DataObject.class.php
 * \brief Classe de base pour la gestion d'entit en base de donnes
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 17.02.2006
 * 
 *
 * Cette classe fournit des mthodes de base pour la gestion d'entit
 * stockes en base de donnes (initialisation, sauvegarde, suppression...)
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once("Helpers.class.php");

class DataObject {
  protected $id;
  protected $errorMsg = null;
  protected $db;

  protected $statusTypes = array( 0 => "Dsactiv",
							1 => "Activ"
							);

  // Donnes de pagination
  protected $displayItems;
  protected $currentPage;
  protected $pageNbr;
  protected $fields;
  protected $from;
  protected $cond;
  protected $order;

  protected $totalRecords;
  
  public $data;
  //

  /**
   * \brief Constructeur
   * \param id integer (optionnel) Numro d'id d'une entit existante avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
    $this->db = DatabasePool::getInstance();

    if ($id) {
      $this->id = $id;
    }
  }

  /**
   * \brief Mthode renvoyant le numro d'identifiant de l'entit en cours
   * \return l'id en cours ou null si non dfinit
  */
  public function getId() {
    return (isset($this->id)) ? $this->id : null;
  }

  /**
   * \brief Mthode permettant de fixer l'identifiant de l'entit en cours
   * \param $pId integer Numro d'identifiant de l'entit
  */
  public function setId($pId) {
    $this->id = $pId;
  }

  /**
   * \brief Mthode permettant de fixer la valeur d'un attribut
   * \param $name chane : Nom de l'attribut
   * \param $val : valeur de l'attribut
  */
  public function set($name, $val) {
	$this->$name = $val;
  }

  /**
   * \brief Mthode permettant de rcuprer la valeur d'un attribut
   * \param $name chane : Nom de l'attribut
   * \return Valeur de l'attribut ou null si l'attribut n'existe pas
  */
  public function get($name) {
    return (isset($this->$name)) ? $this->$name : null;
  }

  /**
   * \brief Mthode permettant de dterminer si l'entit est un nouvel enregistrement ou non
    * \return true si nouvel enregistrement, false sinon
  */
  public function isNew() {
	if (! isset($this->id) ||empty($this->id)) {
	  return true;
	} else {
	  return false;
	}
  }

  /**
   * \brief Mthode initialisant l'entit avec l'identifiant courant
   * \return true si succs, false sinon
  */
  public function init() {
  	
	if (isset($this->id) && ! empty($this->id)) {
	  $sql = "SELECT " . implode(array_keys($this->dbFields), ", ") . " FROM " . $this->objectName . " WHERE id='" . $this->id . "'";
	
	  $result = $this->db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() == 1) {
		$row = $result->get_next_row();
	  
		foreach (array_keys($this->dbFields) as $key) {
		  $this->$key = Helpers::getFromBDD($row[$key]);
		}

		return true;
	  } else {
		$this->errorMsg = "::init - Rsultat incorrect pour l'initialisation de l'entit";
		return false;
	  }
	}

    return false;
  }

  /**
   * \brief Mthode de suppression d'une entit dans la base de donnes
   * \param $id integer (optionnel) Numro d'identifiant de l'identit, si non spcifi, entit en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && is_numeric($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

    $sql = "DELETE FROM " . $this->objectName . " WHERE id='" . $id . "'";

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression de l'entit d'identifiant " . $id;
	  return false;
    }

    return true;
  }

  /**
   * \brief Mthode permettant de valider les attributs de l'entit courante (bon type, prsence...)
   * \return true si succs, false sinon
  */
  public function validate() {
	$this->errorMsg = "";

	foreach ($this->dbFields as $name => $attr) {
	  if (! empty($attr["mandatory"]) && (! isset($this->$name) || strlen($this->$name) <= 0)) {
		$this->errorMsg .= $attr["descr"] . " doit tre prsent.\n";
	  } else {
		if (isset($this->$name) && ! empty($this->$name)) {
		  if (! empty($attr["unique"]) && ! $this->checkUnicity($name)) {
			  $this->errorMsg .= $attr["descr"] . " doit tre unique.\n";
		  } else {
			switch ($attr["type"]) {
			case "isInt":
			  if (! preg_match("/^[-+]?[0-9]+$/", $this->$name)) {
				$this->errorMsg .= $attr["descr"] . " doit tre un entier.\n";
			  }
			  break;
			case "isFloat":
			  if (! preg_match ("/^[-+]?[0-9]+(\.[0-9]+)*$/", $this->$name)) {
				$this->errorMsg .= $attr["descr"] . " doit tre un rel.\n";
			  }
			  break;
			case "isEmail":
			  if (! preg_match ("/^[^@]+@[^@]+\.[a-zA-Z]{2,4}$/", $this->$name)) {
				$this->errorMsg .= $attr["descr"] . " doit tre une adresse lectronique valide.\n";
			  }
			  break;
			}
			
			if (isset($attr["maxlength"])) {
			  if (strlen($this->$name) > $attr["maxlength"]) {
				$this->errorMsg .= "Le champ " . $attr["descr"] . " est trop long (" . $attr["maxlength"] . " caractres maxi autoriss).\n";
			  }
			}
			
			if (isset($attr["regexp"])) {
			  if (! preg_match($attr["regexp"], $this->$name)) {
				$this->errorMsg .= "Le champ " . $attr["descr"] . " " . $attr["regexp_txt"];
			  }
			}
		  }
		}
	  }
	}

	if (! empty($this->errorMsg)) {
	  return false;
	} else {
	  return true;
	}
  }

  /**
   * \brief Mthode permettant de vrifier qu'un attribut est unique dans une table
   * \param $name chane : Nom de l'attribut
   * \return true si unique, false sinon
  */
  public function checkUnicity($name) {
	$sql = "SELECT * FROM " . $this->objectName . " WHERE " . $name . "='" . $this->$name . "'";

	if (isset($this->id)) {
	  $sql .= " AND id != " . $this->id;
	}

	$result = $this->db->select($sql);

	if (! $result->isError()) {
	  if ($result->num_row() > 0) {
		return false;
	  } else {
		return true;
	  }
	} else {
	  return false;
	}
  }

  /**
   * \brief Mthode d'enregistrement d'une entit dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (true par dfaut)
   * \param $return_rather_than_exec boolen (optionnel) Si true, renvoi la requte SQL, si false excute la requte (false par dfaut)
   * \return true si succs, false sinon
  */
  public function save($validate = true, $return_rather_than_exec = false) {
      
    $new = true;
    if (isset($this->id)) {
      $new = false;
    }

    $sql = "";

	if ($validate) {
	  if (! $this->validate()) {
		return false;
	  }
	}

    if ($new) {
      if (! ($this->id = $this->getNextId())) {
		$this->errorMsg = "Erreur de rcupration du nouvel ID";		
		return false;
      }
 

      $sql = "INSERT INTO " . $this->objectName . " (id, ";
      $sql .= implode(array_keys($this->dbFields), ", ");
      $sql .= ") VALUES (" . $this->id;

      foreach($this->dbFields as $field => $val) {
		$sql .= ", ";
		$sql .= (isset($this->$field) && strlen($this->$field) > 0) ? "'" . addslashes($this->$field) . "'" : "NULL";
      }

      $sql .= ")";
    } else { // Mise  jour
      $sql = "UPDATE " . $this->objectName . " SET ";
      
      $fields = array();
      foreach ($this->dbFields as $field => $val) {
		$str = $field . "=";
		$str .= (isset($this->$field) && strlen($this->$field) > 0) ? "'" . addslashes($this->$field) . "'" : "NULL";
		$fields[] = $str;
      }

      $sql .= implode($fields, ", ");

      $sql .= " WHERE id='". $this->id . "'";
    }


    if ($return_rather_than_exec) {
      return $sql;
    } else {
      if (! $this->db->exec($sql)) {
		$this->errorMsg = "Erreur lors de la sauvegarde de l'entit";
		return false;
      }
    }

    return true;
  }

  /**
   * \brief Mthode de rcupration du prochain identifiant dans la base de donnes pour cette entit
   * \return l'identifiant ou null si chec
  */
  protected function getNextId() {
    $sql = "SELECT nextval('" . $this->objectName . "_id_seq') AS id";

    $result = $this->db->select($sql);

    if (! $result->isError()) {
      $row = $result->get_next_row();

      return $row["id"];
    }

    return null;
  }

  /**
   * \brief Mthode d'obtention de la chaine  passer en paramtre  la fonction de validation javascript des formulaires
   * \param .. Un nombre variable de chanes reprsentant les noms des variables membres  valider
   * \return La chane de description des champs  valider
  */
  public function getValidationTrio() {
	$args = func_get_args();

	$ret = array();

	for ($i = 0; $i < count($args); $i++) {
	  if (isset($this->dbFields[$args[$i]])) {
		$str = "'" . addslashes($args[$i]) . "', '" . addslashes($this->dbFields[$args[$i]]["descr"]) . "', '";

		if ($this->dbFields[$args[$i]]["mandatory"]) {
		  $str .= "R";
		}

		$str .= addslashes($this->dbFields[$args[$i]]["type"]);

		if (! empty($this->dbFields[$args[$i]]["maxlength"])) {
		  $str .= "maxLength" . $this->dbFields[$args[$i]]["maxlength"] . "!";
		}

		if (! empty($this->dbFields[$args[$i]]["regexp"])) {
		  // On enlve les caractres / au dbut et  la fin de la regexp, javascript les rajoute automatiquement
		  $str .= "RegExp" . substr($this->dbFields[$args[$i]]["regexp"], 1, -1) . "#";
		}

		$str .= "'";

		$ret[] = $str;
	  }
	}

	return implode($ret, ',');
  }

  // Mthodes de pagination


  /**
   * \brief Mthode d'initialisation du pager
   * \param $fields chane : Liste des champs  rcuprer
   * \param $from chane : Liste des tables dans lesquelles rcuprer les donnes (peut inclure des jointures)
   * \param $cond chane (optionnel) : Chane conditionnelle (comporte une instruction WHERE)
   * \param $order chane (optionnel) : Chane dfinissant l'ordre de tri pour les rsultats (SQL)
   * \param $count entier (optionnel) : Nombre de rsultat par page dsirs
   * \param $page entier (optionnel) : Page courante dsire
   * \return True en cas de succs, false sinon
  */
  public function pagerInit($fields, $from, $cond = null, $order = null, $count = null, $page = null,$sortWay="DESC") {
	$this->fields = $fields;
	$this->from = $from;
	$this->cond = $cond;

	$this->order = $this->objectName . ".id";
	

	if ($order) {
	  $this->order = $order;
	} elseif (isset($_GET["order"])) {
	  // On vrifie que ce champ est bien prsent dans la table concerne
	  if (array_search($_GET["order"], array_keys($this->dbFields)) !== false) {
		$this->order = $this->objectName . "." . $_GET["order"];
	  }	 
	  if (isset($_GET["sortway"])) {
		if ($_GET["sortway"] == "asc") {
		  $sortWay = " ASC";
		}
	  }
	}

	$this->order .= " ".$sortWay;

	if ($count) {
	  $this->displayItems = $count;
	} elseif (isset($_GET["count"]) && is_numeric($_GET["count"])) {
	  $this->displayItems = $_GET["count"];
	} else {
	  $this->displayItems = DEFAULT_ITEMS_PER_PAGE;
	}

	if ($page) {
	  $this->currentPage = $page;
	} elseif (isset($_GET["page"]) && is_numeric($_GET["page"])) {
	  $this->currentPage = $_GET["page"];
	} else {
	  $this->currentPage = 1;
	}

	if (! $this->pagerCountRecords()) {
	  $this->errorMsg = "Erreur lors du comptage des enregistrements.";
	  return false;
	} else {
	  if (! $this->pagerFetchData()) {
		$this->errorMsg = "Erreur lors de la rcupration des enregistrements.";
		return false;
	  }
	}

	return true;
  }

  /**
   * \brief Mthode de comptage du nombre total d'enregistrements pour le pager
   * \return True en cas de succs, false sinon
  */
  protected function pagerCountRecords() {
	// Comptage du nombre total d'enregistrements
	  $sql = "SELECT count(*) AS total_count FROM " . $this->from . " " . $this->cond;

    $db = DatabasePool::getInstance();

    $result = $db->select($sql);

    if (! $result->isError())
    {
      $row = $result->get_next_row();

		  // Nombre total d'enregistrements
		  $this->totalRecords = $row["total_count"];
	
		  // Nombre de pages en fonction du nombre d'items par page
		  $this->pageNbr = ceil($this->totalRecords / $this->displayItems);
	
		  // Controle du dbordement des pages
		  if ($this->currentPage > $this->pageNbr) 
		  {
			   $this->currentPage = $this->pageNbr;
	    }
    }
    else {
	   return false;
	  }

	return true;
 }

  /**
   * \brief Mthode de rcupration des donnes pour le pager depuis la base de donnes
   * \return True en cas de succs, false sinon
  */
  protected function pagerFetchData() {
	$offset = ($this->currentPage - 1) * $this->displayItems;
	if ($offset < 0){
		$offset = 0;	
	}
	$sql = "SELECT " . $this->fields . " FROM " . $this->from . " " . $this->cond;

	if (! empty($this->order)) {
	  $sql .= " ORDER BY " . $this->order;
	}

	$sql .= " LIMIT " . $this->displayItems . " OFFSET " . $offset;

	//echo $sql;

    $db = DatabasePool::getInstance();

    $result = $db->select($sql);

    if (! $result->isError()) {
	  $this->data = $result->get_all_rows();
    } else {
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode de rcupration du message d'erreur associ  l'entit
   * \return Le message d'erreur
  */
  public function getErrorMsg() {
    $msg = $this->errorMsg;
    $this->errorMsg = null;

    return $msg;
  }
}
