<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
/**
 * \class ActesBatch ActesBatch.class.php
 * \brief Cette classe permet de grer les lots de transactions ACTES
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 02.02.2007
 * 
 *
 * Cette classe fournit des mthodes de gestion des lots de transactions
 * Actes
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once (SITEROOT . "/class/DataObject.class.php");
require_once (SITEROOT . "/public.ssl/modules/actes/class/ActesBatchFile.class.php");

class ActesBatch extends DataObject {
  protected $objectName = "actes_batches";

  protected $description;
  protected $storage_dir;
  protected $submission_date;
  protected $user_id;
  protected $num_prefix;
  protected $next_suffix;

  protected $dbFields = array (
    "description" => array (
      "descr" => "Intitul du lot",
      "type" => "isString",
      "mandatory" => true
    ),
    "storage_dir" => array (
      "descr" => "Rpertoire de stockage sur le serveur",
      "type" => "isString",
      "mandatory" => false
    ),
    "submission_date" => array (
      "descr" => "Date de soumission du lot",
      "type" => "isDate",
      "mandatory" => true
    ),
    "user_id" => array (
      "descr" => "Utilisateur propritaire du lot",
      "type" => "isInt",
      "mandatory" => true
    ),
    "num_prefix" => array (
      "descr" => "Prfixe numro interne transactions",
      "type" => "isString",
      "mandatory" => false
    ),
    "next_suffix" => array (
      "descr" => "Prochain suffixe du numro interne",
      "type" => "isInt",
      "mandatory" => false
    )
  );

  protected $batchFiles;
  protected $unprocessedBatchFiles;

  /**
   * \brief Constructeur d'un lot
   * \param id integer Numro d'identifiant d'un lot existant avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
    parent :: __construct($id);
  }

  /**
   * \brief Mthode initialisant l'entit avec l'identifiant courant
   * \return true si succs, false sinon
  */
  public function init() {
    return $this->initBatchFiles() && parent :: init();
  }

  /**
   * \brief Initialisation du rpertoire de stockage (cration)
   * \return True en cas de succs, false sinon
   */
  public function initStorage() {
    if (isset ($this->user_id) && isset ($this->id)) {
      $owner = new User($this->user_id);
      $owner->init();
      $authority = new Authority($owner->get("authority_id"));
      $authority->init();

      $this->storage_dir = $authority->get("siren") . "/" . $this->id;

      if ($this->storage_dir == "/" || !Helpers :: createDirTree(ACTES_BATCHES_UPLOAD_ROOT . "/" . $this->storage_dir, ACTES_BATCHES_UPLOAD_ROOT)) {
        $this->errorMsg = "Erreur de cration du rpertoire de stockage du lot.";
        return false;
      }
    } else {
      return false;
    }

    return true;
  }

  /**
   * \brief Nettoyage du rpertoire de stockage
   * \return True en cas de succs, false sinon
   */
  public function purgeStorage() {
    foreach ($this->batchFiles as $batchFile) {
      if (!$batchFile->deleteFile()) {
        return false;
      }
    }

    if (!Helpers :: deleteFromFS(ACTES_BATCHES_UPLOAD_ROOT . "/" . $this->storage_dir)) {
      return false;
    }

    return true;
  }

  /**
   * \brief Suppression du rpertoire de stockage
   * \return True en cas de succs, false sinon
   */
  public function deleteStorage() {
    return Helpers :: deleteFromFS(ACTES_BATCHES_UPLOAD_ROOT . "/" . $this->storage_dir);
  }

  /**
   * \brief Initialisation des fichiers associs  un lot
   */
  public function initBatchFiles() {
    if (isset ($this->id)) {
      $this->batchFiles = array ();

      $fileIds = ActesBatchFile :: getFilesIdForBatch($this->id);

      foreach ($fileIds as $fileId) {
        $batchFile = new ActesBatchFile($fileId);

        if ($batchFile->init()) {
          $this->batchFiles[] = $batchFile;
        }
      }

      return true;
    } else {
      return false;
    }
  }

  /**
   * \brief Rcupration des fichiers associs  un lot
   * \return Un tableau de ActesBatchFile associ au lot
   */
  public function getBatchFiles() {
    if (!isset ($this->batchFiles)) {
      $this->initBatchFiles();
    }

    return $this->batchFiles;
  }

  /**
   * \brief Mthode d'obtention de la liste des lots et tous leurs attributs
   * \param $cond (optionnel) chane : Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \return Tableau des lots
   */
  public function getBatchesList($cond = "") {
    if (!$this->pagerInit('DISTINCT actes_batches.id, actes_batches.user_id, actes_batches.submission_date, actes_batches.storage_dir, actes_batches.num_prefix, actes_batches.description', 'actes_batches', $cond)) {
      return false;
    }

    return $this->data;
  }

  /**
   * \brief Mthode d'obtention de la liste des lots et tous leurs attributs pour un utilisateur
   * \param $user_id integer : Identifiant de l'utilisateur
   * \return Tableau des lots
   */
  public function getBatchesListForUser($user_id) {
    if (is_numeric($user_id)) {
      return $this->getBatchesList(" WHERE actes_batches.user_id=" . $user_id);
    }

    return array ();
  }

  /**
   * \brief Mthode de rafraichissement de la liste des fichiers du lot restant  traiter
   * \return Tableau des fichiers restant  traiter
   */
  private function refreshUnprocessedFiles() {
    if (!isset ($this->batchFiles)) {
      $this->initBatchFiles();
    }

    $this->unprocessedBatchFiles = array ();

    foreach ($this->batchFiles as $file) {
      if (!$file->isProcessed()) {
        $this->unprocessedBatchFiles[] = $file;
      }
    }
  }

  /**
   * \brief Mthode d'obtention de la liste des fichiers du lot restant  traiter
   * \return Tableau des fichiers restant  traiter
   */
  public function getUnprocessedFiles() {
    if (!isset ($this->unprocessedBatchFiles)) {
      $this->refreshUnprocessedFiles();
    }

    return $this->unprocessedBatchFiles;
  }

  /**
   * \brief Mthode d'obtention du nombre de fichiers du lot restant  traiter
   * \return Le nombre de fichier restant  traiter dans le lot
   */
  public function getUnprocessedFilesCount() {
    if (!isset ($this->unprocessedBatchFiles)) {
      $this->refreshUnprocessedFiles();
    }

    return count($this->unprocessedBatchFiles);
  }

  /**
   * \brief Mthode d'obtention du nombre de fichiers du lot
   * \return Le nombre de fichier dans le lot
   */
  public function getAllFilesCount() {
    if (!isset ($this->batchFiles)) {
      $this->initBatchFiles();
    }

    return count($this->batchFiles);
  }

  /**
   * \brief Mthode d'obtention de la liste des fichiers du lot restant  traiter sous forme id/name
   * \return Tableau des fichiers restant  traiter, clef=id, valeur=nom fichier
   */
  public function getUnprocessedFilesIdName() {
    if (!isset ($this->unprocessedBatchFiles)) {
      $this->refreshUnprocessedFiles();
    }

    return $this->getFilesIdName($this->unprocessedBatchFiles);
  }

  /**
   * \brief Mthode d'obtention de la liste des fichiers du lot sous forme id/name
   * \return Tableau des fichiers, clef=id, valeur=nom fichier
   */
  public function getAllFilesIdName() {
    if (!isset ($this->batchFiles)) {
      $this->initBatchFiles();
    }

    return $this->getFilesIdName($this->batchFiles);
  }

  /**
   * \brief Mthode d'obtention de la liste de fichiers du lot sous forme id/name
   * \param $files Tableau d'objet ActesBatchFiles
   * \return Tableau des fichiers, clef=id, valeur=nom fichier
   */
  private function getFilesIdName($files) {
    $retFiles = array ();

    if (is_array($files)) {
      foreach ($files as $file) {
        $retFiles[$file->getId()] = basename($file->get("filename"));
      }
    }

    return $retFiles;
  }

  /**
   * \brief Rcupration du prochain Id de fichier  traiter
   * \param $currentId entier (optionnel) : identifiant du fichier courant
   * \return L'identifiant du prochain fichier  traiter ou null si plus de fichier  traiter
   */
  public function getNextUnprocessedId($currentId = null) {
    $nextId = null;

    $this->initBatchFiles();

    $this->refreshUnprocessedFiles();
    $files = $this->getUnprocessedFilesIdName();

    if (count($files) > 0) {
      $fileId = array_keys($files);

      $pos = 0;
      $newPos = 0;
      if ($currentId) {
        $pos = array_search($currentId, $fileId);

        if ($pos !== false) {
          $newPos = ($pos +1) % count($fileId);
        }
      }

      if ($currentId && $newPos == $pos) {
        $nextId = null;
      } else {
        $nextId = $fileId[$newPos];
      }
    }

    return $nextId;
  }

  /**
   * \brief Rcupration du prochain suffixe pour le numro interne
   * \return Le suffixe pour le numro interne
   */
  public function getNextSuffix() {
    if (isset ($this->id)) {
      $next_suffix = 1;

      // On rcupre le suffixe
      $sql = "SELECT next_suffix FROM actes_batches WHERE id=" . $this->id . " FOR UPDATE";

      $result = $this->db->select($sql);

      if (!$result->isError()) {
        if ($result->num_row() > 0) {
          $row = $result->get_next_row();
          if (is_numeric($row["next_suffix"])) {
            $next_suffix = $row["next_suffix"];
          }
        }
      } else {
        return false;
      }
      return $next_suffix;
    } else {
      return null;
    }
  }

  /**
   * \brief Incrmentaiton du prochain suffixe pour le numro interne
   */
  public function incNextSuffix() {
    if (isset ($this->id)) {
      $next_suffix = $this->getNextSuffix();

      if ($next_suffix != null) {
        $sql = "UPDATE actes_batches SET next_suffix=" . ($next_suffix +1) . " WHERE id=" . $this->id;

        if (!$this->db->exec($sql)){
          $this->errorMsg = "Erreur d'accs base de donnes.";
          return $this->errorMsg;
        }
        return $this->getNextSuffix();
      } else
        $this->errorMsg = "Impossible de rcuprer le suffixe courant.";
        return $this->errorMsg;
    }
  }

  /**
   * \brief Mthode d'importation des fichiers depuis les parametres du formulaire
   * \param $files tableau des fichiers (format $_FILE)
   * \return True en cas de succs ou False sinon
   */
  public function importFilesFromForm($files) {
    $ret_value = true;

    if (is_array($files)) {

      // Copie du tableau pour recherche des signatures
      $filesDup = $files;

      $this->errorMsg = "<span style='font-weight:bold;color:red;'>Echec de la cr&eacute;ation du lot.</span><br />\n";

      reset($files);
      while ((list ($key, $file) = each($files))) {
        
      	$filename=str_replace("\'","-",$file['name']);
      	$filename=str_replace("'","-",$filename);
		if (strstr($filename,"?"))
		{	
			$this->errorMsg .= "Le fichier " . $filename . " ne doit pas contenir des lettres accentues";
			$ret_value = false;	
		}
      	//echo $fiename;
        //$filename = Helpers :: stripSlashes($file["name"]);

        // On ne traite pas individuellement les fichiers de signature
        if (!preg_match("/\.sig$/", $filename)) {
          if (is_uploaded_file($file["tmp_name"])) {
            // Vrification du type de fichier
            $type = Helpers :: getFileType($file["tmp_name"]);
          	$trace = Trace::getInstance();
          	
          	$trace->log("Rcuperation de ". $file['name'] . " - type : " .$type  . " - tmp_name : " . $file["tmp_name"]);
          	
          	
            if ( $type != "application/pdf") {
              $this->errorMsg .= "Le fichier <span style='font-weight:bold;'>" . $filename . "</span> n'est pas du type requis ($type). <span style='font-weight:bold;'>Seuls les fichiers PDF sont autoris&eacute;s.</span><br />\n";
              $ret_value = false;
            } else {
              // On recherche un fichier de signature associ
              $foundSig = false;
              $sign = "";
              reset($filesDup);
            }

            if ($ret_value) {
              if (!$this->addBatchFile($file["tmp_name"], $filename, $sign)) {
                $ret_value = false;
              }
            }
          } else {
            $this->errorMsg .= "Envoi de fichier incorrect.<br />";
            $ret_value = false;
          }
        }
      }
    } else {
      $ret_value = false;
    }

    return $ret_value;
  }

  /**
   * \brief Mthode d'ajout d'un fichier dans le lot
   * \param $filepath chemin vers le fichier dans le systme de fichier
   * \param $name nom du fichier final
   * \param $sign chaine : signature lectronique du fichier
   * \return True en cas de succs ou False sinon
   */
  public function addBatchFile($filepath, $name, $sign) {
    $batchF = new ActesBatchFile();

    $batchF->set("signature", $sign);
    $batchF->set("tmpName", $name);
    $batchF->set("tmpFile", $filepath);
    $batchF->setUnprocessed();

    $this->batchFiles[] = $batchF;

    return true;
  }

  /**
   * \brief Mthode d'enregistrement d'un lot dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (true par dfaut)
   * \return true si succs, false sinon
   */
  public function save($validate = true) {
    $new = false;
    if ($this->isNew()) {
      $new = true;

      if (empty ($this->description)) {
        $this->description = "Lot post le " . date("d-m-Y");
      }

      $this->submission_date = date("Y-m-d H:i:s");
    }

    if (!($sql = parent :: save($validate, true))) {
      return false;
    }

    if ($new) {
      // Si nouveau lot, cration rpertoire de stockage
      if (!$this->initStorage()) {
        return false;
      }
    }

    //echo $sql;
    //exit();

    if (!$this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
    }

    if (!$this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde du lot.";
      $this->db->rollback();
      return false;
    }

    // Dfinition du rpertoire de stockage
    if (!$this->db->exec("UPDATE actes_batches SET storage_dir='" . $this->storage_dir . "' WHERE id=" . $this->id)) {
      $this->errorMsg = "Erreur lors de la dfinition du rpertoire de stockage du lot.";
      $this->db->rollback();
      return false;
    }

    foreach ($this->batchFiles as $batchFile) {
      if ($new) {
        $batchFile->set("batch_id", $this->id);
        $batchFile->set("storage_dir", $this->storage_dir);
      }

      if (!$batchFile->save()) {
        $this->errorMsg = "Erreur d'enregistrement d'un fichier du lot : " . $batchFile->getErrorMsg();
        $this->db->rollback();

        if ($new) {
          $this->purgeStorage();
        }

        return false;
      }
    }

    if (!$this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
      $this->db->rollback();

      if ($new) {
        $this->purgeStorage();
      }

      return false;
    }

    return true;
  }

  /**
   * \brief Mthode de suppression d'un lot
   * \return true si succs, false sinon
  */
  public function delete() {
    if (!$this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
    }

    // Suppression des fichiers inclus
    if (!isset ($this->batchFiles)) {
      $this->initBatchFiles();
    }

    foreach ($this->batchFiles as $batchFile) {
      if (!$batchFile->delete()) {
        $this->errorMsg = "Erreur lors de la suppression des fichiers associs au lot : " . $batchFile->getErrorMsg();
        $this->db->rollback();
        return false;
      }
    }

    // Suppression du rpertoire de stockage
    if (!$this->deleteStorage()) {
      return false;
    }

    if (!parent :: delete()) {
      return false;
    }

    if (!$this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
      $this->db->rollback();
      return false;
    }

    return true;
  }
}
?>
