<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/



/**
 * \class MailUtil.class.php
 * \brief fonction commun pour envoiyer les email.
 * 
 * Cette classe fournit des mthodes d'envois email de module mail.
 *
 * \author TH ,JMontiel
 * \date :23-04-2008
 * 
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once (SITEROOT . '/class/include.class.php');
require_once (SITEROOT."/class/pearMail.class.php");


class MailUtil {
 	
	public $errorMsg;
	private $trace;
	private $subject;
	private $from;
	
	public function __construct(){
		$this->trace = Trace::getInstance();
		$this->setSubjet(MAIL_MESSAGE);
		$this->setFrom(MAIL_TEDETIS_FROM);	
	}
	
	/**
	 * \brief	Permet de vrifier qu'un repertoire ne contient pas
	 * 			de fichier infest par un virus.
	 * 
	 * \param	$path string le chemin du repertoire  analyser
	 * 
	 * \return 	true : le rpertoire ne contient pas de fichier infecter,
	 * 		 	false sinon (fichier infect, impossible d'appeller la commande
	 * 		 	$this->errorMsg contient le message d'erreur
	 * 
	 */
 	public function checkSanity($path){
 		
 		Trace::wrap_exec(ANTIVIRUS_COMMAND . " " . $path, $output, $ret);
		
		switch ($ret) {
			
	  		case 0:
				return true;
				break;
				
	  		case 1:
				$this->errorMsg = "L'archive est infecte par un virus. Retour de l'antivirus&nbsp;:<br />\n";
				// Format de ligne : /Nom/de/fichier: Nom virus
				foreach ($output as $line) {
			  		if (preg_match('/^\/.*: .* FOUND$/', $line)) {
						$line = explode(":", $line);
						$this->errorMsg .= basename($line[0]) . " : " . $line[1] . "<br />\n";
		  			}
				}
				return false;
				break;
				
	  		default:
				$this->errorMsg = "Erreur " . $ret . " lors du scan antivirus de l'archive.";
				return false;
	  		}
 	}
 	
 	/**
 	 * \brief extrait un tableau de fichier  partir d'un repertoire
 	 * \param $root le chemin racine
 	 * \return array le tableau des fichiers (n'inclue pas les repertoire)
 	 */
	public function path2array( $root) {
    	$result = array();
    	$dh = @opendir( $root );
    	if( false === $dh ) {
        	return $result;
    	}
    
    	while( $file = readdir( $dh )) {
        	if( "." == $file || ".." == $file ){
            	continue;
        	}
        	
        	if( is_dir( $root."/".$file )) {
            	$result += $this->path2array( $root."/".$file);
        	} else {
        		array_push($result,$root."/".$file);
        	}
    	}
    	closedir( $dh );
    	return $result;
	}
 	
 	/**
 	 * \brief 	zip les fichiers dont le chemin est dans array_path et 
 	 * 			met le rsultat dans $file
 	 * \param	array_file array tableau des fichiers  zipper
 	 * \param	$file string nom du fichier de sortie
 	 * \return	true si ok false sinon. $this->errorMsg contient le message d'erreur
 	 */
 	public function zip($array_file, $file){
		$zip = new ZipArchive();
		$res = $zip->open($file, ZipArchive::CREATE);
		
		if (! $res){
			$this->errorMsg = "Impossible de crer l'archive zip $file : erreur ".$res;
			$this->trace->log($this->errorMsg,Trace::$TRACE_ERROR);
			return false;
		} 
		
		foreach ($array_file as $fichier) {
			if (!$zip->addFile($fichier, basename($fichier))) {
				$this->errorMsg = "Impossible de mettre le fichier $fichier dans l'archive $file ";
			 	$this->trace->log($this->errorMsg,Trace::$TRACE_ERROR);
			 	return false;
			}
		}
		$zip->close();
		return true;
 	}


 
 	
 	public function setSubjet($subject){
 		$this->subject = $subject; 		
 	}
 	
 	public function setFrom($from){
 		if ($from){
 			$this->from = $from;
 		}
 	}
 	
 	/**
   * \brief   envoyer un mail avec des pices joindures.
   * \param   objet du class : mail_message_emis
   * \param                    mail_transaction
   * \param                    mail_include_file
   * \param   
   * \return  true si ok false sinon.
   */    
	public function sendMail($MailMessageEmis,$mailTransaction,$MailIncludeFiles,$send_password = false)
	{
	  	 $from = $this->from;
	  	 $from = "-f{$from}";  
	  	 
		
		$text=MAIL_TEXT;
		if ($mailTransaction->getPassword() ) {
			$text.="\n\n";
			if ($send_password) {
				$text.="Le mot de passe du document est : ".$mailTransaction->getPassword()."\n";
			} else {
				$text.="Le document est protg par un mot de passe";
			}
		}
		

		$html='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 TRANSITIONAL//EN">
<html>
  <body bgcolor="#ffffff" text="#000000">
';
		$html.= "<p>".nl2br($text)."</p>";

		$hdrs = array(
              	'From'    => $this->from,
              	'Subject' => $this->subject,
				'Reply-To' => $this->from,
				'Return-path' => $this->from
              );
             
             
		    	
	  	foreach ($MailMessageEmis as $MailEmis )
   		{
	    	if (defined('MAIL_DEBUG'))
	    	{
				echo "<p>mail file number =".$MailFileNumber;"</p>";	
				echo "$MailEmis->getEmail()";
	    	}	
	    	$htmlpart='';	
			$htmlpart.='
<a href="'.WEBSITE.'/modules/mail/?command=show&mail_emis_id='.$MailEmis->getId().'" >Confirmer la reception et lire le courrier en cliquant sur ce lien</a><br>
<p>Information de scurit : tous les documents ont t tests par l\'anti-virus CLAMAV.</p>
<p>Pour toute demande d\'information, vous pouvez contacter l\'expditeur prcis dans le contenu du message sur la plateforme scurise.</p>
';

	    	$htmlpart.='';
			
			$textpart=WEBSITE."/modules/mail/index.php?command=show&mail_emis_id=".$MailEmis->getId();
			$textpart.="\nInformation de scurit : tous les documents ont t tests par l'anti-virus CLAMAV.\n";
			
			$crlf="\n";
			$mime = new Mail_mime($crlf);
			$mime->setTXTBody($text.$textpart);
			$mime->setHTMLBody($html.$htmlpart);
			
			//do not ever try to call these lines in reverse order
			$body = $mime->get();
			$hdrs = $mime->headers($hdrs);
                        
                        $tomime = new Mail_mime($crlf);
                        $tohdrs = array('To' => $MailEmis->getEmail());
                        $tohdrs = $tomime->headers($tohdrs);
                        $to = $tohdrs['To'];
			
			$mail =new pearMail();
			$mail->sep = $crlf;
  			if (!$mail->send($to, $hdrs, $body,$from)) {
	        		return false;
  			}
    	}	
     return true;
	}

  public function GetMailMessage()
  {
	$mbox = imap_open("{".IMAP_SERVER."/novalidate-cert}",IMAP_LOGIN, IMAP_PASS);
	//echo imap_last_error();
	$headers = imap_headers($mbox);
	
	if ($headers == false) {
	    return false;
	}
	$mailEmis=array("mail_id"=>array(),"mail_emis_id"=>array());
  	$mail='mail_id=';
	$emis="mail_emis_id=";
	$mailLength=strlen($mail);
	$emisLength=strlen($emis);
	for($x=1; $x <=count($headers); $x++) {
		$mailBody=imap_body($mbox,$x);
		$mailId=strstr($mailBody,$mail);
		if ($mailId !=null)
		{
			$mailEmisId=strstr($mailId,$emis);
			if ($mailEmisId!=null)
			{		
				$mailEmisId=substr($mailEmisId,$emisLength,strlen($mailEmisId)-strlen(strstr($mailEmisId,"-"))-$emisLength);
				$mailId=substr($mailId,$mailLength,strlen($mailId)-strlen(strstr($mailId,"&"))-$mailLength);
				$mailEmis["mail_id"][]=$mailId;
				$mailEmis["mail_emis_id"][]=$mailEmisId;
				$mailEmis["body"][]=$mailBody;
				if (defined('MAIL_DEBUG'))
		    	{
					echo "<p>mail_id = $mailId     mail_emis_id= $mailEmisId </p>";
				}
			}
		}	
		imap_delete($mbox,$x);
	}
	imap_expunge($mbox);
	imap_close($mbox);
	return $mailEmis;
   }

}
